import { _decorator, Component, EventTouch, UITransform, Vec2, Vec3, Node, v3, view, find } from "cc";
const { ccclass, property } = _decorator;

@ccclass('Joystick')
export class Joystick extends Component {
    @property(Node)
    joystickBg: Node = null!;  // 摇杆背景大圆
    @property(Node)
    stick: Node = null!;       // 摇杆控制小圆点
    //@property
    maxRadius: number = 80;    // 摇杆拖拽最大半径
    @property(Node)
    gameRoot: Node = null!;      // 画布节点（触摸监听目标）

    private _dir: Vec3 = Vec3.ZERO;
    private _isTouching: boolean = false;
    private _uiTransform: UITransform = null!;  // 
    private _defaultBgPos: Vec3 = new Vec3();   // 摇杆背景默认位置

    get direction(): Vec3 { return this._dir; }
    get isActive(): boolean { return this._isTouching; }

    onLoad() {
        
        // 缓存画布的UITransform组件
        this._uiTransform = this.gameRoot.getComponent(UITransform)!;
        // 记录摇杆背景初始位置（如屏幕边缘）
        this._defaultBgPos = this.joystickBg.position.clone();

        // 在画布上监听触摸事件（非摇杆自身）
        this.gameRoot.on(Node.EventType.TOUCH_START, this.onTouchStart, this);
        this.gameRoot.on(Node.EventType.TOUCH_MOVE, this.onTouchMove, this);
        this.gameRoot.on(Node.EventType.TOUCH_END, this.onTouchEnd, this);
        this.gameRoot.on(Node.EventType.TOUCH_CANCEL, this.onTouchEnd, this);
    }

    private onTouchStart(event: EventTouch) {
        this._isTouching = true;
        
        // 将摇杆背景移动到触摸点位置
        const touchPos = this.convertTouchToCanvasSpace(event);
        this.joystickBg.setPosition(touchPos);
        this.stick.setPosition(Vec3.ZERO);  // 小圆点复位到中心
        
        // 立即触发一次方向计算
        this.updateStickPosition(event);
    }

    private onTouchMove(event: EventTouch) {
        if (!this._isTouching) return;
        this.updateStickPosition(event);
    }

    private onTouchEnd() {
        this._isTouching = false;
        // 复位摇杆背景到默认位置（如屏幕边缘）
        this.joystickBg.setPosition(this._defaultBgPos);
        this.stick.setPosition(Vec3.ZERO);
        this._dir = Vec3.ZERO;
    }

    /** 核心逻辑：更新摇杆小圆点位置并计算方向 */
    private updateStickPosition(event: EventTouch) {
        // 获取触摸点相对于摇杆背景的坐标
        const touchPos = this.convertTouchToCanvasSpace(event);
        const bgPos = this.joystickBg.position;
        const delta = new Vec3(touchPos.x - bgPos.x, touchPos.y - bgPos.y, 0);

        // 限制拖拽半径
        const distance = Vec3.len(delta);
        if (distance > this.maxRadius) {
            Vec3.multiplyScalar(delta, delta, this.maxRadius / distance);
        }
        this.stick.setPosition(delta);

        // 归一化方向向量（若在死区内则归零）
        this._dir = distance > 5 ? Vec3.normalize(new Vec3(), delta) : Vec3.ZERO;
    }

    /** 将触摸点转换为画布空间坐标 */
    private convertTouchToCanvasSpace(event: EventTouch): Vec3 {
        const screenPos = event.getUILocation();
        // 将屏幕坐标转换为画布节点本地坐标
        return this._uiTransform.convertToNodeSpaceAR(v3(screenPos.x, screenPos.y, 0));
    }
}
