import { _decorator, BoxCollider2D, Collider2D, Component, Contact2DType, IPhysics2DContact, math, RigidBody2D, toRadian, tween, Vec3, Vec2, PhysicsSystem2D, Sprite } from "cc";
let { ccclass, property } = _decorator;
@ccclass('Weapon')
export class Weapon extends Component {
    // 物理组件声明
    @property(RigidBody2D)
    rigidBody!: RigidBody2D;
    @property(BoxCollider2D)
    collider!: BoxCollider2D;
    private _orbitRadius: number = 100;
    private _baseSpeed: number = 1;
    private _currentAngle: number = 0;
    // 击飞方向
    private moveDirection: Vec2 = Vec2.ZERO;
    // 新增可配置参数
    //@property({ tooltip: "是否自动更新角度" })
    autoUpdateAngle: boolean = true;

    //@property({ tooltip: "角度偏移量（度）" })
    angleOffset: number = 90;
    // 击飞速度
    //@property
    private knockBackSpeed: number = 800;

    // 旋转速度（度/秒）
    //@property
    private rotateSpeed: number = 720;
    // 运行时变量优化
    private _posCache = new Vec3();
    private _currentRadian: number = 0;
    _faction = 0;
    private _offset = new Vec3();
    // 参数化初始化
    public async setup(config: {
        faction: number,
        skin: string,
        orbitRadius: number,
        baseSpeed: number
    }) {
        //this.sprite.spriteFrame =  config.skin;
       
        //this.node.getComponent(Sprite).spriteFrame = sf;
        this._orbitRadius = config.orbitRadius;
        this._baseSpeed = config.baseSpeed;
    }
    protected onLoad(): void {
        this.node.getComponent(Collider2D).on(Contact2DType.BEGIN_CONTACT, this.onBeginContact, this);
        this.node.getComponent(Collider2D).on(Contact2DType.END_CONTACT, this.onEndContact, this);
        //this._offset.set(this.node.position.subtract(this.node.position));
        let a =  PhysicsSystem2D.instance.autoSimulation;
    }
    
    // 状态切换
    public setState(state: 'orbit' | 'flyaway') {
        switch (state) {
            case 'orbit':
                this.enabled = true;
                break;
            case 'flyaway':
                this.enabled = false; // 停止轨道更新
                break;
        }
    }
    // 重置状态（回收时调用）
    public resetState() {
        this.setState('orbit');
        this._currentAngle = 0;
    }
    // 更新环绕位置
    public updateOrbit(angle: number) {
        this._currentAngle = angle;
        this.updatePosition();
    }
    protected update(dt: number) {
        if (!this.enabled) {
            // 计算位移
            let moveDelta = this.moveDirection
                .multiply(new Vec2(this.knockBackSpeed * dt, this.knockBackSpeed * dt));

            // 更新位置
            this.node.position = new Vec3(
                this.node.position.x + moveDelta.x,
                this.node.position.y + moveDelta.y,
                0
            );

            // 更新旋转（根据方向决定顺时针/逆时针）
            let rotateDir = this.moveDirection.x > 0 ? 1 : -1;
            this.node.angle += this.rotateSpeed * dt * rotateDir;

        } else {
            this._currentAngle -= this._baseSpeed * dt * 60;
            this.updatePosition();
        }

    }
    private updatePosition() {
        this._currentRadian = toRadian(this._currentAngle);

        // 复用向量对象减少GC
        Vec3.set(
            this._posCache,
            Math.cos(this._currentRadian) * this._orbitRadius,
            Math.sin(this._currentRadian) * this._orbitRadius,
            0
        );
        this.node.position = this._posCache;
        // 新增切线角度计算
        if (this.autoUpdateAngle) {
            this.updateBladeAngle();
        }
    }
    // 独立的角度计算方法
    private updateBladeAngle() {
        // 计算切线方向角度（数学推导公式）
        let tangentAngle = math.toDegree(Math.atan2(
            this._posCache.y,
            this._posCache.x
        )) - this.angleOffset;

        this.node.angle = tangentAngle;
    }
    // 新增平滑过渡方法
    public smoothSetOrbit(targetRadius: number, duration: number = 0.5) {
        // 使用Tween实现平滑过渡（需引入tween）
        let current = { val: this._orbitRadius };
        tween(current)
            .to(duration, { val: targetRadius }, {
                onUpdate: () => {
                    this._orbitRadius = current.val;
                },
                easing: 'sineOut'
            })
            .start();
    }
    // 碰撞响应处理
    onBeginContact(selfCollider: Collider2D, otherCollider: Collider2D, contact: IPhysics2DContact | null) {
        console.log('onBeginContact 111');
        if (!this.enabled) return
        // 1. 获取实际碰撞点
        let collisionPoint: Vec3;
        if (contact) {
            let manifold = contact.getWorldManifold();
            if (manifold.points.length > 0) {
                // 使用第一个接触点的世界坐标
                let worldPoint = manifold.points[0];
                collisionPoint = new Vec3(worldPoint.x, worldPoint.y);
            } else {
                // 无接触点时使用包围盒中心
                collisionPoint = this.calculateFallbackPoint(selfCollider, otherCollider);
            }
        } else {
            collisionPoint = this.calculateFallbackPoint(selfCollider, otherCollider);
        }
        // 计算方向：碰撞点指向飞刀中心的向量反方向
        let selfWorldPos = this.node.worldPosition;
        this.moveDirection = new Vec2(
            selfWorldPos.x - collisionPoint.x,
            selfWorldPos.y - collisionPoint.y
        ).normalize();
       // this.collider.enabled = false;
        this.setState('flyaway');
    }
    protected onEndContact(selfCollider: Collider2D, otherCollider: Collider2D, contact: IPhysics2DContact | null) {
        console.log('onEndContact111');
    }
    // 备用计算方案（包围盒中心）
    private calculateFallbackPoint(self: Collider2D, other: Collider2D): Vec3 {
        let selfPos = new Vec3();
        let otherPos = new Vec3();
        self.node.getWorldPosition(selfPos);
        other.node.getWorldPosition(otherPos);
        return new Vec3(
            (selfPos.x + otherPos.x) * 0.5,
            (selfPos.y + otherPos.y) * 0.5
        );
    }
}
